<?php


namespace Tfaf\Inc\Api\Callbacks;

require_once TFAF_PLUGIN_PATH . 'Inc/Base/BaseController.php';

class CmeCallbacks
{

    /**
     * Callback for Custom Menu Entry Section
     */
    public function cmeSection()
    {
        echo 'Manage your custom menu type.';
    }

    /**
     * Callback for Custom Menu Entry Section for editing a single Entry
     */
    public function cmeSectionManager()
    {
        echo 'Create a new custom menu entry. You can change the position of the new entry at the overview after adding it.';
    }

    /**
     * Sanitize the user input and add the input to the options array
     *
     * @param $input single menu entry
     *
     * @return array with all menu entries
     */
    public function cmeSanitize($input)
    {

        // get saved menu entries
        $output = get_option('tfaf_custom_menu');

        // delete menu entry
        if (isset($_POST["remove"])) {
            unset($output[$_POST["remove"]]);
            $ordered_output[1] = "";
            // reorder positions
            $keys = array_keys($output);
            for ($position = 1; $position <= count($output); $position++) {
                $ordered_output[$position] = $output[$keys[$position - 1]];
                $ordered_output[$position]['menu_position'] = $position;
            }

            return $ordered_output;
        }

        // change menu positions
        if (isset($_POST["positions"])) {

            // check if something was changed
            if ($_POST["positions"] == "") {
                return $output;
            }

            $positions = sanitize_text_field($_POST["positions"]);

            $positions = explode("-", $positions);
            unset($positions[count($positions) - 1]);

            // validate keys
            $positions_validate = array_diff_key($output, array_flip($positions));
            if (!empty($positions_validate)) {
                add_settings_error('tfaf_custom_menu', esc_attr('tfaf_custom_menu'),
                    esc_html__('Invalid Position.', 'toolbox-for-asgaros-forum'), 'error');

                return $output;
            }


            $ordered_output[1] = "";
            for ($i = 0; $i < count($positions); $i++) {
                $ordered_output[$i + 1] = $output[$positions[$i]];
                $ordered_output[$i + 1]["menu_position"] = ($i + 1) . "";
            }
            return $ordered_output;
        }

        // sanitize and validate input
        $input['menu_position'] = (int)$input['menu_position'];

        $max_position = count($output) + 1;

        if ($input['menu_position'] < 1 || $input['menu_position'] > $max_position) {
            add_settings_error('tfaf_custom_menu', esc_attr('tfaf_custom_menu'),
                esc_html__('Invalid Position.', 'toolbox-for-asgaros-forum'), 'error');

            return $output;
        }


        $input['menu_link_text'] = sanitize_text_field($input['menu_link_text']);
        if ($input['menu_url'] !== esc_url_raw($input['menu_url'])) {
            add_settings_error('tfaf_custom_menu', esc_attr('tfaf_custom_menu'),
                esc_html__('Invalid URL. URL must start with https:// or http://', 'toolbox-for-asgaros-forum'), 'error');

            return $output;
        } else {
            $input['menu_url'] = esc_url_raw($input['menu_url']);
        }

        if (isset($input['menu_new_tab'])) {
            $input['menu_new_tab'] = sanitize_text_field($input['menu_new_tab']);
        }

        if (isset($input['menu_only_users'])) {
            $input['menu_only_users'] = sanitize_text_field($input['menu_only_users']);
        }

        if (count($output) == 0) {
            $output[$input['menu_position']] = $input;

            return $output;
        }

        foreach ($output as $key => $value) {
            if ($input['menu_position'] === $key) {
                $output[$key] = $input;
            } else {
                $output[$input['menu_position']] = $input;
            }
        }

        return $output;
    }

    /**
     * Callback for a textfield
     *
     * @param $args arguments for Text Field
     */
    public function textField($args)
    {
        $name = $args['label_for'];
        $option_name = $args['option_name'];
        $value = '';
        $disabled = '';

        // Check if a existing entry has to be edit
        if (isset($_POST["edit_entry"])) {
            $edit_cme = sanitize_text_field($_POST["edit_entry"]);
            $input = get_option($option_name);

            // Check if key is valid
            if (array_key_exists($edit_cme, $input)) {
                $value = $input[$edit_cme][$name];
            } else {
                $value = esc_html__('Error: Entry can\'t be edited.', 'toolbox-for-asgaros-forum');
            }

            $disabled = ($name == 'menu_position') ? 'readonly' : '';
        }

        echo '<input type="text" class="regular-text" id="' . $name . '" name="' . $option_name . '[' .
            $name . ']" value="' . $value . '" placeholder="' . $args['placeholder'] . '" ' . $disabled . ' required>';
    }

    /**
     * Callback for the position field
     *
     * @param $args arguments for position Field
     */
    public function position($args)
    {
        $name = $args['label_for'];
        $option_name = $args['option_name'];
        $input = get_option($option_name);
        $count = count($input) + 1;

        // Check if a existing entry has to be edit
        if (isset($_POST["edit_entry"])) {
            $count = sanitize_text_field($_POST["edit_entry"]);

            // check if position is valid
            if (!array_key_exists($count, $input)) {
                $count = esc_html__('Error: Position is not valid.', 'toolbox-for-asgaros-forum');
            }
        }

        echo '<input type="text" class="regular-text" id="' . $name . '" name="' . $option_name . '[' .
            $name . ']" value="' . $count . '" readonly required>';

    }

    /**
     * Callback for Url Field
     *
     * @param $args argument for url Field
     */
    public function urlField($args)
    {
        $name = $args['label_for'];
        $option_name = $args['option_name'];
        $value = '';

        // Check if a existing entry has to be edit
        if (isset($_POST["edit_entry"])) {
            $edit_cme = sanitize_text_field($_POST["edit_entry"]);
            $input = get_option($option_name);

            // Check if key is valid
            if (array_key_exists($edit_cme, $input)) {
                $value = $input[$edit_cme][$name];
            } else {
                $value = esc_html__('Error: Entry can\'t be edited.', 'toolbox-for-asgaros-forum');
            }
        }

        echo '<input type="text" class="regular-text" id="' . $name . '" name="' . $option_name . '[' .
            $name . ']" value="' . $value . '" placeholder="' . $args['placeholder'] . '" pattern="(https?:/)?/.+" required>';

    }

    /**
     * Callback for a checkbox field
     *
     * @param $args arguments for a checkbox field
     */
    public function checkboxField($args)
    {
        $name = $args['label_for'];
        $classes = $args['class'];
        $option_name = $args['option_name'];
        $checked = false;

        // Check if a existing entry has to be edit
        if (isset($_POST["edit_entry"])) {
            $checkbox = get_option($option_name);
            $checked = isset($checkbox[$_POST["edit_entry"]][$name]) ?: false;
        }


        echo '<div class="' . $classes . '"><input type="checkbox" id="' . $name . '" name="' . $option_name . '[' .
            $name . ']" value="1" class="" ' . ($checked ? 'checked' : '') . '><label for="' . $name .
            '"><div></div></label></div>';
    }
}